<?php
/**
 * @package   AdminTools
 * @copyright 2010-2021 Akeeba Ltd / Nicholas K. Dionysopoulos
 * @license   GNU General Public License version 3, or later
 */

namespace Joomla\Plugin\Actionlog\AdminTools\Extension;

defined('_JEXEC') or die;

use Akeeba\Component\AdminTools\Administrator\Controller\DatabasetoolsController;
use Akeeba\Component\AdminTools\Administrator\Model\DatabasetoolsModel;
use Akeeba\Component\AdminTools\Administrator\Model\ScanalertsModel;
use Joomla\CMS\Component\ComponentHelper;
use Joomla\CMS\MVC\Controller\BaseController;
use Joomla\CMS\User\User;
use Joomla\Component\Actionlogs\Administrator\Plugin\ActionLogPlugin;
use Joomla\Event\SubscriberInterface;
use ReflectionMethod;

class AdminTools extends ActionLogPlugin implements SubscriberInterface
{
	/**
	 * Load the language file on instantiation.
	 *
	 * @var    boolean
	 * @since  3.1
	 */
	protected $autoloadLanguage = true;

	/**
	 * Application object.
	 *
	 * @var    \Joomla\CMS\Application\CMSApplication
	 * @since  3.7.0
	 */
	protected $app;

	private $defaultExtension = 'com_admintools';

	/**
	 * Returns an array of events this subscriber will listen to.
	 *
	 * @return  array
	 *
	 * @since   9.0.0
	 */
	public static function getSubscribedEvents(): array
	{
		// Only subscribe events if the component is installed and enabled
		if (!ComponentHelper::isEnabled('com_akeebabackup'))
		{
			return [];
		}

		// Register all public onSomething methods as event handlers
		$events   = [];
		$refClass = new \ReflectionClass(__CLASS__);
		$methods  = $refClass->getMethods(ReflectionMethod::IS_PUBLIC);

		foreach ($methods as $method)
		{
			$name = $method->getName();

			if (substr($name, 0, 2) != 'on')
			{
				continue;
			}

			$events[$name] = $name;
		}

		return $events;
	}

	public function onComAdmintoolsQuickstartControllerAfterCommit($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_QUICKSTART_SAVE', 'com_admintools');
	}

	public function onComAdmintoolsAdminpasswordControllerBeforeProtect($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_ADMINPASSWORD_ENABLE', 'com_admintools');
	}

	public function onComAdmintoolsAdminpasswordControllerBeforeUnprotect($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_ADMINPASSWORD_DISABLE', 'com_admintools');
	}

	public function onComAdmintoolsMainpasswordControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_MASTERPASSWORD_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsMainpasswordControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_MASTERPASSWORD_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsEmergencyofflineControllerBeforeOffline($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_EMERGENCYOFFLINE_ENABLE', 'com_admintools');
	}

	public function onComAdmintoolsEmergencyofflineControllerBeforeOnline($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_EMERGENCYOFFLINE_DISABLE', 'com_admintools');
	}

	public function onComAdmintoolsCleantempdirectoryControllerBeforeMain($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CLEANTEMPDIRECTORY_RUN', 'com_admintools');
	}

	/**
	 * @param   DatabasetoolsController  $controller
	 */
	public function onComAdmintoolsDatabasetoolsControllerAfterOptimize($controller)
	{
		/** @var DatabasetoolsModel $model */
		$model   = $controller->getModel();
		$percent = $model->getState('percent', 0);

		if ($percent >= 100)
		{
			$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_DATABASETOOLS_REPAIR', 'com_admintools');
		}
	}

	public function onComAdmintoolsDatabasetoolsControllerBeforePurgesessions($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_DATABASETOOLS_PURGESESSIONS', 'com_admintools');
	}

	public function onComAdmintoolsChecktempandlogdirectoriesControllerBeforeCheck($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CHECKTEMPANDLOGDIRECTORIES_RUN', 'com_admintools');
	}

	public function onComAdmintoolsConfigurewafControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CONFIGUREWAF_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsConfigurewafControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CONFIGUREWAF_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsExportimportControllerBeforeDoexport($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_IMPORANDEXPORT_EXPORT', 'com_admintools');
	}

	public function onComAdmintoolsExportimportControllerBeforeDoimport($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_IMPORANDEXPORT_IMPORT', 'com_admintools');
	}

	public function onComAdmintoolsSeoandlinktoolsControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_SEOANDLINKTOOLS_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsSeoandlinktoolsControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_SEOANDLINKTOOLS_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsFixpermissionsControllerBeforeMain($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_FIXPERMISSIONS_RUN', 'com_admintools');
	}

	public function onComAdmintoolsConfigurepermissionsControllerAfterSavedefaults($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CONFIGUREFIXPERMISSIONS_DEFAULTS', 'com_admintools');
	}

	public function onComAdmintoolsConfigurepermissionsControllerBeforeSaveperms($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CONFIGUREFIXPERMISSIONS_SAVEPERMS', 'com_admintools');
	}

	public function onComAdmintoolsConfigurepermissionsControllerBeforeSaveapplyperms($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_CONFIGUREFIXPERMISSIONS_SAVEAPPLYPERMS', 'com_admintools');
	}

	public function onComAdmintoolsHtaccessmakerControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_HTACCESSMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsHtaccessmakerControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_HTACCESSMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsNginxconfmakerControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_NGINXCONFMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsNginxconfmakerControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_NGINXCONFMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsWebconfigmakerControllerAfterSave($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_WEBCONFIGMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsWebconfigmakerControllerAfterApply($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_WEBCONFIGMAKER_EDIT', 'com_admintools');
	}

	public function onComAdmintoolsScansControllerBeforeStartscan($controller)
	{
		$this->logUserAction('', 'COM_ADMINTOOLS_LOGS_SCANS_RUN', 'com_admintools');
	}

	public function onComAdmintoolsScanalertsControllerAfterPublish($controller)
	{
		/** @var ScanalertsModel $model */
		$model = $controller->getModel();
		$ids   = $this->getIDsFromRequest();

		if (!$ids)
		{
			return;
		}

		$table = $model->getTable('Scanalert', 'Administrator');

		foreach ($ids as $id)
		{
			if (!$table->load($id))
			{
				continue;
			}

			$this->logUserAction($table->path, 'COM_ADMINTOOLS_LOGS_SCANALERTS_MARKEDSAFE', 'com_admintools');
		}
	}

	public function onComAdmintoolsScanalertsControllerAfterUnpublish($controller)
	{
		/** @var ScanalertsModel $model */
		$model = $controller->getModel();
		$ids   = $this->getIDsFromRequest();

		if (!$ids)
		{
			return;
		}

		$table = $model->getTable('Scanalert', 'Administrator');

		foreach ($ids as $id)
		{
			if (!$table->load($id))
			{
				continue;
			}

			$this->logUserAction($table->path, 'COM_ADMINTOOLS_LOGS_SCANALERTS_MARKEDUNSAFE', 'com_admintools');
		}
	}

	public function onComAdmintoolsScanalertsControllerAfterMarkallsafe($controller)
	{
		$scan_id = $this->app->input->getInt('scan_id', 0);

		if (empty($scan_id))
		{
			return;
		}

		$this->logUserAction($scan_id, 'COM_ADMINTOOLS_LOGS_SCANALERTS_MARKEDALLSAFE', 'com_admintools');
	}

	/* Start of CRUD tasks */

	public function onComAdmintoolsBadwordControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'word', 'COM_ADMINTOOLS_LOGS_BADWORDS_EDIT');
	}

	public function onComAdmintoolsBadwordControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'word', 'COM_ADMINTOOLS_LOGS_BADWORDS_EDIT');
	}

	public function onComAdmintoolsBadwordControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'word', 'COM_ADMINTOOLS_LOGS_BADWORDS_EDIT');
	}

	public function onComAdmintoolsBadwordsControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'word', 'COM_ADMINTOOLS_LOGS_BADWORDS_DELETE');
	}

	public function onComAdmintoolsWafdenylistControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_EDIT');
	}

	public function onComAdmintoolsWafdenylistControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_EDIT');
	}

	public function onComAdmintoolsWafdenylistControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_EDIT');
	}

	public function onComAdmintoolsWafdenylistsControllerAfterPublish($controller)
	{
		$this->logCRUDAction($controller, 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_PUBLISH');
	}

	public function onComAdmintoolsWafdenylistsControllerAfterUnpublish($controller)
	{
		$this->logCRUDAction($controller, 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_UNPUBLISH');
	}

	public function onComAdmintoolsWafdenylistsControllerAfterDelete($controller)
	{
		$this->logCRUDAction($controller, 'id', 'COM_ADMINTOOLS_LOGS_WAFBLACKLIST_DELETE');
	}

	public function onComAdmintoolsWafexceptionControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFEXCEPTIONS_EDIT');
	}

	public function onComAdmintoolsWafexceptionControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFEXCEPTIONS_EDIT');
	}

	public function onComAdmintoolsWafexceptionControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'id', 'COM_ADMINTOOLS_LOGS_WAFEXCEPTIONS_EDIT');
	}

	public function onComAdmintoolsWafexceptionsControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'id', 'COM_ADMINTOOLS_LOGS_WAFEXCEPTIONS_DELETE');
	}

	public function onComAdmintoolsAdminallowlistControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_WHITELISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsAdminallowlistControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_WHITELISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsAdminallowlistControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_WHITELISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsAdminallowlistsControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'ip', 'COM_ADMINTOOLS_LOGS_WHITELISTEDADDRESSES_DELETE');
	}

	public function onComAdmintoolsIPDenyListControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_BLACKLISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsIPDenyListControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_BLACKLISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsIPDenyListControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_BLACKLISTEDADDRESSES_EDIT');
	}

	public function onComAdmintoolsIPDenyListsControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'ip', 'COM_ADMINTOOLS_LOGS_BLACKLISTEDADDRESSES_DELETE');
	}

	public function onComAdmintoolsBlockedrequestslogControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'ip', 'COM_ADMINTOOLS_LOGS_SECURITYEXCEPTIONS_DELETE');
	}

	public function onComAdmintoolsBlockedrequestslogControllerBeforeBan($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_SECURITYEXCEPTIONS_BAN');
	}

	public function onComAdmintoolsBlockedrequestslogControllerBeforeUnban($controller)
	{
		$this->logCRUDSave($controller, 'id', 'ip', 'COM_ADMINTOOLS_LOGS_SECURITYEXCEPTIONS_BAN');
	}

	public function onComAdmintoolsAutobannedaddressesControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'ip', 'COM_ADMINTOOLS_LOGS_AUTOBANNEDADDRESSES_DELETE');
	}

	public function onComAdmintoolsIpautobanhistoriesControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'ip', 'COM_ADMINTOOLS_LOGS_IPAUTOBANHISTORIES_DELETE');
	}

	public function onComAdmintoolsUrlredirectionControllerAfterApply($controller)
	{
		$this->logCRUDSave($controller, 'id', 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_EDIT');
	}

	public function onComAdmintoolsUrlredirectionControllerAfterSave($controller)
	{
		$this->logCRUDSave($controller, 'id', 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_EDIT');
	}

	public function onComAdmintoolsUrlredirectionControllerAfterSave2new($controller)
	{
		$this->logCRUDSave($controller, 'id', 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_EDIT');
	}

	public function onComAdmintoolsUrlredirectionsControllerAfterPublish($controller)
	{
		$this->logCRUDAction($controller, 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_PUBLISH');
	}

	public function onComAdmintoolsUrlredirectionsControllerAfterUnpublish($controller)
	{
		$this->logCRUDAction($controller, 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_UNPUBLISH');
	}

	public function onComAdmintoolsUrlredirectionsControllerBeforeDelete($controller)
	{
		$this->logCRUDAction($controller, 'dest', 'COM_ADMINTOOLS_LOGS_REDIRECTIONS_DELETE');
	}

	/* End of CRUD tasks */

	/**
	 * Gets the list of IDs from the request data
	 *
	 * @return array
	 */
	private function getIDsFromRequest()
	{
		// Get the ID or list of IDs from the request or the configuration
		$cid = $this->app->input->get('cid', [], 'array');
		$id  = $this->app->input->getInt('id', 0);

		$ids = [];

		if (is_array($cid) && !empty($cid))
		{
			$ids = $cid;
		}
		elseif (!empty($id))
		{
			$ids = [$id];
		}

		return $ids;
	}

	/**
	 * Log a user action.
	 *
	 * This is a simple wrapper around self::addLog
	 *
	 * @param   string|array  $title               Language key for title or an array of additional data to record in
	 *                                             the audit log.
	 * @param   string        $messageLanguageKey  Language key describing the user action taken.
	 * @param   string|null   $context             The name of the extension being logged (default: use
	 *                                             $this->defaultExtension).
	 * @param   User|null     $user                User object taking this action (default: currently logged in user).
	 *
	 * @return  void
	 *
	 * @see     self::addLog
	 * @since   9.0.0
	 */
	private function logUserAction($title, string $messageLanguageKey, ?string $context = null, ?User $user = null): void
	{
		// Get the user if not defined
		$user = $user ?? $this->app->getIdentity();

		// No log for guests
		if (empty($user) || ($user->guest))
		{
			return;
		}

		// Default extension if none defined
		$context = $context ?? $this->defaultExtension;

		$message = [
			'username'    => $user->username,
			'accountlink' => 'index.php?option=com_users&task=user.edit&id=' . $user->id,
		];

		if (!is_array($title))
		{
			$title = [
				'title' => $title,
			];
		}

		$message = array_merge($message, $title);

		$this->addLog([$message], $messageLanguageKey, $context, $user->id);
	}

	/**
	 * @param           $controller
	 * @param   string  $primaryKey
	 * @param   string  $displayKey
	 * @param   string  $translationKey
	 */
	private function logCRUDSave(BaseController $controller, string $primaryKey, string $displayKey, string $translationKey): void
	{
		$model          = $controller->getModel();
		$controllerName = $controller->getName();
		$table          = $model->getTable($controllerName, 'Administrator');

		if (!$table->load($this->app->input->getInt($primaryKey)))
		{
			return;
		}

		$link = sprintf(
			"<a href=\"index.php?option=com_admintools&view=%s&task=edit&%s=%s\">%s</a>",
			urlencode($controllerName),
			urlencode($primaryKey),
			urlencode($table->{$primaryKey}),
			$model->{$displayKey});

		$this->logUserAction($link, $translationKey, 'com_admintools');
	}

	/**
	 * @param           $controller
	 * @param   string  $displayKey
	 * @param   string  $translationKey
	 */
	private function logCRUDAction($controller, string $displayKey, string $translationKey): void
	{
		$ids   = $this->getIDsFromRequest();
		$model = $controller->getModel();
		$table = $model->getTable($controller->getName(), 'Administrator');

		foreach ($ids as $id)
		{
			if (!$table->load($id))
			{
				continue;
			}

			$this->logUserAction($table->{$displayKey}, $translationKey, 'com_admintools');
		}
	}

}